c
      real*4 function brntop(ax,bx,cx,f,xvlo,xvec,xvhi,
     +                       xshf,nvec,tol,xmin)
c
c brntop: Use Brent's method to find bracketed minimum
c
c Update history:
c    V1.0  18-Nov-1992  Original version (from Press et al BRENT)
c    V2.0  16-Jun-1994  Changed name, tidied up
c    V2.1  20-Jul-1998  Slight improvement
c
      implicit none
c
c Arguments
c    ax      = r*4          = first bracketing point
c    bx      = r*4          = second bracketing point
c    cx      = r*4          = third bracketing point
c    f       = r*4 function = function to minimize
c    xvlo(i) = r*4(nvec)    = lower limits of parameters
c    xvec(i) = r*4(nvec)    = vector of parameters
c    xvhi(i) = r*4(nvec)    = upper limits of parameters
c    xshf(i) = r*4(nvec)    = vector of shift amount
c    nvec    = i*4          = length of parameters
c    tol     = r*4          = fractional accuracy requested
c    xmin    = r*4          = amount of xshf to get to min
c
      integer*4 nvec
      real*4    ax,bx,cx,tol,xmin
      real*4    f
      external  f
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec),xshf(nvec)
c
c Local parameters
c    NVMAX = i*4 = maximum length of vectors
c    ITMAX = i*4 = maximum number of iterations
c    CGOLD = r*4 = golden ratio factor
c    ZEPS  = r*4 = accuracy
c
      integer*4  ITMAX,NVMAX
      real*4     CGOLD,ZEPS
      parameter (ITMAX=100)
      parameter (NVMAX=128)
      parameter (CGOLD=0.3819660)
      parameter (ZEPS =1.0e-10)
c
c Local variables
c    a       = r*4        = smallest bracket value
c    b       = r*4        = largest bracket value
c    d       = r*4        = distance to move (parabolic interpolation)
c    e       = r*4        = change in value
c    etemp   = r*4        = temporary storage for e
c    fu      = r*4        = function value at x
c    fv      = r*4        = function value at x
c    fw      = r*4        = function value at x
c    fx      = r*4        = function value at x
c    i1      = i*4        = counting index
c    iter    = i*4        = iteration count
c    p       = r*4        = interpolation variable
c    q       = r*4        = interpolation variable
c    r       = r*4        = interpolation variable
c    tol1    = r*4        = local tolerance
c    tol2    = r*4        = local tolerance
c    u       = r*4        = new attempt at minimum
c    v       = r*4        = changing upper limit
c    w       = r*4        = changing bracketed value
c    x       = r*4        = changing bracketed value
c    xm      = r*4        = range midpoint
c    xoff(i) = r*4(NVMAX) = offset point for function calculation
c
      real*4    a,b,d,e,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,
     +          u,v,w,x,xm
      real*4    xoff(NVMAX)
      integer*4 i1,iter
c
c Initialise location information, putting a,b in ascending order
c
      a=min(ax,cx)
      b=max(ax,cx)
      v=bx
      w=v
      x=v
c
c Get distance moved on penultimate step
c
      e=0.0
c
c Get function value at point nearest minimum
c
      do i1=1,nvec
        xoff(i1)=max(xvlo(i1),min(xvec(i1)+x*xshf(i1),xvhi(i1)))
      enddo
      fx=f(xoff,nvec)
c
c Initialise other function values
c
      fv=fx
      fw=fx
c
c Main loop: done ITMAX times at most
c
      do iter=1,ITMAX
c
        xm  =0.5*(a+b)
        tol1=tol*abs(x)+ZEPS
        tol2=2.0*tol1
c
c Test for convergence: if done, return with results
c
        if (abs(x-xm) .le. (tol2-0.5*(b-a))) then
          xmin  =x
          brntop=fx
          return
        endif
c
c Not done, construct trial parabolic fit to values of
c    function
c
        if (abs(e) .gt. tol1) then
c
          r=(x-w)*(fx-fv)
          q=(x-v)*(fx-fw)
          p=(x-v)*q-(x-w)*r
          q=2.0*(q-r)
c
          if (q .gt. 0.0) then
            p=-p
          endif
c
          q    =abs(q)
          etemp=e
          e    =d
c
c Check acceptability of parabolic fit
c
          if (abs(p) .ge. abs(0.5*q*etemp) .or.
     +        p      .le. q*(a-x)          .or. 
     +        p      .ge. q*(b-x)               ) then
            goto 1
          endif
c
c Fit OK, calculate and take parabolic step
c
          d=p/q
          u=x+d
c
          if (u-a .lt. tol2 .or. 
     +        b-u .lt. tol2      ) then
            d=sign(tol1,xm-x)
          endif
c
c Skip over golden section step
c
          goto 2
c
        endif
c
c Perform a golden section step
c
 1      continue
c
c Take step into larger of two segments
c
        if (x .ge. xm) then
          e=a-x
        else
          e=b-x
        endif
c
c Evaluate step amount
c
        d=CGOLD*e
c
c Apply step amount, d, to get new guess, u
c
 2      continue
        if (abs(d) .ge. tol1) then
          u=x+d
        else
          u=x+sign(tol1,d)
        endif
c
c Evaluate function at new location
c
        do i1=1,nvec
          xoff(i1)=max(xvlo(i1),min(xvec(i1)+u*xshf(i1),xvhi(i1)))
        enddo
        fu=f(xoff,nvec)
c
c Decide what to do with result. If improved, then rearrange the
c    endpoints for another round. 
c
        if (fu .le. fx) then
c
          if (u .ge. x) then
            a=x
          else
            b=x
          endif
c
          v =w
          fv=fw
          w =x
          fw=fx
          x =u
          fx=fu
c
        else
c
c Not improved, so a different rearrangement will be needed
c
          if (u .lt. x) then
            a=u
          else
            b=u
          endif
c
          if (fu .le. fw .or. 
     +        w  .eq. x       ) then
            v =w
            fv=fw
            w =u
            fw=fu
          elseif (fu .le. fv .or.
     +            v  .eq. x  .or. 
     +            v  .eq. w       ) then
            v =u
            fv=fu
          endif
c
        endif
c
c Do another iteration
c
      enddo
c
c End of process: all iterations done
c
      xmin  =x
      brntop=fx
c
      end
