c
      subroutine gripo(func,xvlo,xvec,xvhi,nvec)
c
c gripo: Grid + Powell minimization of func
c
c Update history
c    V1.0   3-Aug-1993  Original version
c    V1.1  10-Jul-1998  Small updates
c    V1.2  17-Jul-1998  Fix notational horror
c
      implicit none
c
c Arguments
c    func = r*4 function = quantity to minimize
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c
      integer*4 nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
c
c Subprogrammes
c    getop = subroutine = get controlling parameter
c
      external getop
c
c Local parameter
c    NVMAX = i*4 = maximum size of vectors that can be dealt with
c
      integer*4  NVMAX
      parameter (NVMAX =128)
c
c Local variables
c    fmin    = r*4        = minimum of function
c    fval    = r*4        = current function value
c    i1      = i*4        = counting index
c    i2      = i*4        = counting index
c    i3      = i*4        = counting index
c    iloop   = i*4        = counter of loops
c    indx    = i*4        = counter of varying parameters
c    iprint  = i*4        = print level
c    isum    = i*4        = counter in parameter indexing
c    ivec    = i*4        = index of position in vector
c    ninc    = i*4        = number of values for each parameter
c    nla     = i*4        = actual number of loops to use
c    nloop   = i*4        = number of grid values to try
c    nvar    = i*4        = number of parameters that vary
c    xinc(i) = r*4(NVMAX) = vector of increments in parameters
c    xini(i) = r*4(NVMAX) = initial guess vector
c    xmin(i) = r*4(NVMAX) = current minimum vector
c    value   = r*4        = value of parameter
c
      real*4    fmin,fval,value
      real*4    xinc(NVMAX),xini(NVMAX),xmin(NVMAX)
      integer*4 i1,i2,i3,iloop,indx,iprint,isum,ivec
      integer*4 ninc,nla,nloop,nvar
c
c Get value of controlling parameter
c
      call getop('GRIPO.NLOOP',value)
      nloop=ifix(value+0.5)
      call getop('GRIPO.IPRINT',value)
      iprint=ifix(value+0.5)
c
c Title
c
      if (iprint .gt. 0) then
        write (6,600)
      endif
c
c Check that vector length isn't too large
c
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
c
c Get number of times each variable will be incremented. Do by
c    counting only those variables for which xvlo =/= xvhi, and
c    set to a minimum of two values per variable.
c
      nvar=0
      do i1=1,nvec
        if (xvlo(i1) .ne. xvhi(i1)) then
          nvar=nvar+1
        endif
      enddo
c
      ninc=ifix(10.0**(alog10(float(nloop))/float(nvar)))
      if (ninc .le. 1) then
        ninc=2
      endif
c
c Calculate actual number of loops to use, and comment on the
c    need to truncate at the maximum permitted number (nloop)
c    if too large.
c
      nla=ninc**nvar
      if (nla .gt. nloop) then
        write (6,698) nla,nloop
        nla=nloop
      endif
c
c Get array of increments of each variable
c
      do i1=1,nvec
        xinc(i1)=(xvhi(i1)-xvlo(i1))/float(ninc-1)
      enddo
c
c Get starting value of func, save vector. Save starting vector.
c
      fmin=func(xvec,nvec)
      do i1=1,nvec
        xini(i1)=xvec(i1)
        xmin(i1)=xvec(i1)
      enddo
c
c Show initial value and controlling parameters
c
      if (iprint .gt. 0) then
        write (6,601) nvar,nla,ninc,fmin
      endif
c
c Now, loop over the following nla times. In each loop the variables
c    are assigned within the allowed range.
c
      do iloop=1,nla
c
c For each variable, calculate the corresponding value that it
c    should have in a ninc**nvar hypercubic grid at this value of 
c    iloop. This means that the routine only varies those
c    parameters with xvlo =/= xvhi.
c
        isum=iloop
        indx=0
c
        do ivec=1,nvec
c
          if (xvlo(ivec) .ne. xvhi(ivec)) then
c
            indx=indx+1
            i2  =ninc**(nvar-indx)
            i3  =1+ifix(float(isum-1)/float(i2))
            isum=isum-(i3-1)*(ninc**(nvar-indx))
c
            value=xini(ivec)+float(i3-1)*xinc(ivec)
            if     (value .gt. xvhi(ivec)) then
              value=value-xvhi(ivec)+xvlo(ivec)
            elseif (value .lt. xvlo(ivec)) then
              value=value+xvhi(ivec)-xvlo(ivec)
            endif
c
            xvec(ivec)=value
          endif
c
        enddo
c
c Apply Powell minimization starting at this point
c
        call powll(func,xvlo,xvec,xvhi,nvec)
c
c Save result if a better minimum
c
        fval=func(xvec,nvec)
        if (fval .lt. fmin) then
          fmin=fval
          do i2=1,nvec
            xmin(i2)=xvec(i2)
          enddo
c
c Show progress
c
          if (iprint .gt. 1) then
            write (6,602) fmin,iloop
          endif
c
        endif
c
      enddo
c
c Copy best vector back to xvec
c
      do i1=1,nvec
        xvec(i1)=xmin(i1)
      enddo
c
c End
c
      if (iprint .gt. 0) then
        write (6,603)
      endif
c
 600  format (' gripo: v1.2')
 601  format (' gripo:         no. of variables = ',i8/
     +        '                    no. of loops = ',i8/
     +        '          no. of values/variable = ',i8/
     +        '          initial function value = ',1pe15.5)
 602  format (' gripo:   improved minimum value = ',1pe15.5,
     +        ' at iteration = ',i6)
 603  format (' ')
c
 698  format (' * Warning: gripo: ',i6,' > ',i6,' steps needed:',
     +        ' truncating grid *')
 699  format (' *** Error: gripo: parameter vector too large ***')
c
      end
