c
      subroutine mnbrop(ax,bx,cx,fa,fb,fc,func,
     +                  xvlo,xvec,xvhi,xshf,nvec)
c
c mnbrop: Find three points bracketing a minimum
c
c Method
c    Parabolic interpolation to find minimum. With some protection
c    against overflows
c
c Update history
c    V1.0  prehistory   From Press et al
c    V2.0  13-Oct-1992  Modified to allow constrained searches
c    V3.0  16-Jun-1994  Change name, tidy up
c    V3.1  20-Jul-1998  Small improvements
c
      implicit none
c
c Arguments
c    ax      = r*4          = first point (input and output)
c    bx      = r*4          = second point (input and output)
c    cx      = r*4          = third point (input and output)
c    fa      = r*4          = value of function at ax
c    fb      = r*4          = value of function at bx
c    fc      = r*4          = value of function at cx
c    func    = r*4 function = function to minimise
c    xvlo(i) = r*4(nvec)    = lower limits vector
c    xvec(i) = r*4(nvec)    = vector of parameters
c    xvhi(i) = r*4(nvec)    = upper limits vector
c    xshf(i) = r*4(nvec)    = shift vector of parameters
c    nvec    = i*4          = vector length in parameters
c
      integer*4 nvec
      real*4    ax,bx,cx,fa,fb,fc
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec),xshf(nvec)
c
c Local parameters
c    NVMAX  = i*4 = maximum vector length
c    GOLD   = r*4 = golden ratio
c    GLIMIT = r*4 = maximum shift
c    HUGE   = r*4 = large number to set upper limit of parameters
c    TINY   = r*4 = divide by zero protection
c
      integer*4  NVMAX
      real*4     GOLD,GLIMIT,HUGE,TINY
      parameter (NVMAX =128)
      parameter (GOLD  =1.618034)
      parameter (GLIMIT=100.0)
      parameter (TINY  =1.0e-20)
      parameter (HUGE  =1.0e+10)
c
c Local variables
c    dum     = r*4        = copying variable
c    fu      = r*4        = value of function at new point
c    i1      = i*4        = counting index
c    q       = r*4        = coefficient in parabolic interpolation
c    r       = r*4        = coefficient in parabolic interpolation
c    u       = r*4        = new test value
c    ulim    = r*4        = largest shift allowed in interpolation
c    umax    = r*4        = largest permitted value of shift
c    xoff(i) = r*4(NVMAX) = offset vector for function alculation
c
      integer*4 i1
      real*4    dum,fu,q,r,u,ulim,umax,xoff(NVMAX)
c
c Get values of function at ax and bx
c
      do i1=1,nvec
        xoff(i1)=max(xvlo(i1),min(xvec(i1)+ax*xshf(i1),xvhi(i1)))
      enddo
      fa=func(xoff,nvec)
c
      do i1=1,nvec
        xoff(i1)=max(xvlo(i1),min(xvec(i1)+bx*xshf(i1),xvhi(i1)))
      enddo
      fb=func(xoff,nvec)
c
c Rearrange order, if necessary, so that function is larger at ax
c
      if (fb .gt. fa) then
        dum=ax
        ax =bx
        bx =dum
        dum=fb
        fb =fa
        fa =dum
      endif
c
c Construct a third point and evaluate the function there
c
      cx=bx+GOLD*(bx-ax)
      do i1=1,nvec
        xoff(i1)=max(xvlo(i1),min(xvec(i1)+cx*xshf(i1),xvhi(i1)))
      enddo
      fc=func(xoff,nvec)
c
c Construct a maximum value of u that is to be permitted. This is
c    an uttermost safety limit.
c
      umax=bx+HUGE*(bx-ax)
c
c Main loop: continue around until it is uphill from bx to cx
c
 1    continue
c
      if (fb .ge. fc) then
c
c Parabolic interpolation to construct point u, supposed to be
c    close to a minimum.
c
        r   =(bx-ax)*(fb-fc)
        q   =(bx-cx)*(fb-fa)
        u   =bx-((bx-cx)*q-(bx-ax)*r)/(2.*sign(max(abs(q-r),TINY),q-r))
        ulim=bx+GLIMIT*(cx-bx)
c
        if     ((bx-u)*(u-cx) .gt. 0.0) then
c
          do i1=1,nvec
            xoff(i1)=max(xvlo(i1),
     +                   min(xvec(i1)+u*xshf(i1),xvhi(i1)))
          enddo
          fu=func(xoff,nvec)
c
          if     (fu .lt. fc) then
            ax=bx
            fa=fb
            bx=u
            fb=fu
            goto 1
          elseif (fu .gt. fb) then
            cx=u
            fc=fu
            goto 1
          endif
c
          u=cx+GOLD*(cx-bx)
          do i1=1,nvec
            xoff(i1)=max(xvlo(i1),
     +                   min(xvec(i1)+u*xshf(i1),xvhi(i1)))
          enddo
          fu=func(xoff,nvec)
c
        elseif ((cx-u)*(u-ulim) .gt. 0.0) then
c
          do i1=1,nvec
            xoff(i1)=max(xvlo(i1),
     +                   min(xvec(i1)+u*xshf(i1),xvhi(i1)))
          enddo
          fu=func(xoff,nvec)
c
          if (fu .lt. fc) then
            bx=cx
            cx=u
            u =cx+GOLD*(cx-bx)
            fb=fc
            fc=fu
            do i1=1,nvec
              xoff(i1)=max(xvlo(i1),
     +                     min(xvec(i1)+u*xshf(i1),xvhi(i1)))
            enddo
            fu=func(xoff,nvec)
          endif
c
        elseif ((u-ulim)*(ulim-cx) .ge. 0.0) then
c
          u=ulim
          do i1=1,nvec
            xoff(i1)=max(xvlo(i1),
     +                   min(xvec(i1)+u*xshf(i1),xvhi(i1)))
          enddo
          fu=func(xoff,nvec)
c
        else
c
          u=cx+GOLD*(cx-bx)
          do i1=1,nvec
            xoff(i1)=max(xvlo(i1),
     +                   min(xvec(i1)+u*xshf(i1),xvhi(i1)))
          enddo
          fu=func(xoff,nvec)
c
        endif
c
        ax=bx
        bx=cx
        cx=u
        fa=fb
        fb=fc
        fc=fu
c
c Test that the upper limit hasn't become too large
c
        if (cx .lt. umax) then
          goto 1
        endif
c
      endif
c
c End
c
      end
