*
      subroutine monpo(func,xvlo,xvec,xvhi,nvec)
*
* monpo: Monte-Carlo + Powell minimization of func
*
* Update history
*    v1.0   3-Aug-1993  Original version
*    v1.1  17-Jun-1994  Slight tidying
*    v1.2  20-Jul-1998  New random number generator
*    v1.3  25-Sep-1999  More debug output
*
      implicit none
*
* Arguments
*    func = r*4 function = quantity to minimize
*    xvlo = r*4(nvec)    = array of lower limits to xvec
*    xvec = r*4(nvec)    = array of quantities to find at minimum
*    xvhi = r*4(nvec)    = array of upper limits to xvec
*    nvec = i*4          = number of parameters
*
      integer*4 nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
*
* Subprogrammes
*    getop  = subroutine   = get value of optimization control variable
*    ranuni = r*4 function = random number generator
*
      real*4   ranuni
      external getop,ranuni
*
* Local parameters
*    NVMAX = i*4 = maximum size of vectors that can be dealt with
*
      integer*4  NVMAX
      parameter (NVMAX=128)
*
* Local variables
*    fmin   = r*4        = minimum of function
*    fval   = r*4        = current function value
*    i1     = i*4        = counting index
*    iloop  = i*4        = counter of loops
*    iprint = i*4        = print level
*    iseed  = i*4        = random number generator seed
*    ivec   = i*4        = index of position in vector
*    nloop  = i*4        = number of Monte-Carlo throws to try
*    value  = r*4        = value of variable returned from getop
*    xmin   = r*4(NVMAX) = current minimum vector
*
      real*4    fmin,fval,xmin(NVMAX),value
      integer*4 i1,iloop,iseed,iprint,ivec,nloop
*
* Get controlling parameters
*
      call getop('MONPO.NLOOP',value)
      nloop=ifix(value+0.5)
      call getop('MONPO.IPRINT',value)
      iprint=ifix(value+0.5)
*
* Get random number seed
*
      call getop('RANUNI.ISEED',value)
      iseed=ifix(value+0.5)
*
* Title
*
      if (iprint .gt. 0) then
        write (6,600)
      endif
*
* Check that vector length isn't too large
*
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
*
* Get starting value of func, save vector.
*
      fmin=func(xvec,nvec)
      do i1=1,nvec
        xmin(i1)=xvec(i1)
      enddo
*
* Print stating value
*
      if (iprint .gt. 1) then
        write (6,601) fmin
      endif
*
* Now, loop over the following nloop times. In each loop the variables
*    are assigned within the allowed range.
*
      do iloop=1,nloop
*
* For each variable, calculate the corresponding value using a 
*    uniform distribution in xvlo to xvhi. If xvlo=xvhi, then
*    don't bother.
*
        do ivec=1,nvec
          if (xvlo(ivec) .ne. xvhi(ivec)) then
            xvec(ivec)=xvlo(ivec)+(xvhi(ivec)-xvlo(ivec))*ranuni(iseed)
          endif
        enddo
*
* At this vector, apply Powell minimization
*
        call powll(func,xvlo,xvec,xvhi,nvec)
        fval=func(xvec,nvec)
*
        if (fval .lt. fmin) then
          fmin=fval
          do i1=1,nvec
            xmin(i1)=xvec(i1)
          enddo
*
* Print update
*
          if     (iprint .gt. 2) then
            write (6,602) fmin,iloop
            write (6,603) (xmin(i1), i1=1,nvec)
          elseif (iprint .gt. 1) then
            write (6,602) fmin,iloop
          endif
*
        endif
*
      enddo
*
* Copy minimum back to xvec
*
      do i1=1,nvec
        xvec(i1)=xmin(i1)
      enddo
*
* Save random number seed
*
      value=float(iseed)
      call putop('RANUNI.ISEED',value)
*
* End
*
 600  format (' monpo: v1.3')
 601  format (' monpo: initial function value = ',
     +        1pe15.5)
 602  format (' monpo: improved minimum value = ',
     +        1pe15.5,' at iteration = ',i6)
 603  format (' monpo:        improved vector = ',
     +        4(1pe15.5),31(/' ',32x,4(1pe15.5)))
*
 699  format (' *** Error: monpo: parameter vector too large ***')
*
      end
