c
      subroutine monte(func,xvlo,xvec,xvhi,nvec)
c
c monte: Monte-Carlo minimization of func
c
c Update history:
c    V1.0   8-Oct-1992  Original version
c    V1.1  17-Jun-1994  Slight tidying
c    V1.2  13-Jul-1998  Improved treatment
c
      implicit none
c
c Arguments:
c    func = r*4 function = quantity to minimize
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c
      integer*4 nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
c
c Subprogrammes:
c    getop  = subroutine   = get value of optimization control variable
c    ranuni = r*4 function = random number generator
c
      real*4   ranuni
      external getop,ranuni
c
c Local parameters
c    NVMAX = i*4 = maximum size of vectors that can be dealt with
c
      integer*4  NVMAX
      parameter (NVMAX=128)
c
c Local variables:
c    fmin    = r*4        = minimum of function
c    fval    = r*4        = current function value
c    i1      = i*4        = counting index
c    iloop   = i*4        = counter of loops
c    iprint  = i*4        = print level
c    iseed   = i*4        = random number generator seed
c    ivec    = i*4        = index of position in vector
c    nloop   = i*4        = number of Monte-Carlo throws to try
c    value   = r*4        = value of variable returned from getop
c    xmin(i) = r*4(NVMAX) = current minimum vector
c
      real*4    fmin,fval,xmin(NVMAX),value
      integer*4 i1,iloop,iprint,iseed,ivec,nloop
c
c Get controlling parameters
c
      call getop('MONTE.NLOOP' ,value)
      nloop=ifix(value+0.5)
      call getop('MONTE.IPRINT',value)
      iprint=ifix(value+0.5)
c
c Get random number seed
c
      call getop('RANUNI.ISEED',value)
      iseed=ifix(value+0.5)
c
c Title
c
      if (iprint .gt. 0) then
        write (6,600)
      endif
c
c Check that vector length isn't too large
c
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
c
c Get starting value of func, save vector.
c
      fmin=func(xvec,nvec)
      do i1=1,nvec
        xmin(i1)=xvec(i1)
      enddo
c
c Print stating value
c
      if (iprint .gt. 0) then
        write (6,601) fmin
      endif
c
c Now, loop over the following nloop times. In each loop the variables
c    are assigned within the allowed range.
c
      do iloop=1,nloop
c
c For each variable, calculate the corresponding value using a 
c    uniform distribution in xvlo to xvhi. If xvlo=xvhi, then
c    don't bother.
c
        do ivec=1,nvec
          if (xvlo(ivec) .ne. xvhi(ivec)) then
            xvec(ivec)=xvlo(ivec)+(xvhi(ivec)-xvlo(ivec))*ranuni(iseed)
          endif
        enddo
c
c Get new value of function, save it and the vector if smaller than
c    the previous value.
c
        fval=func(xvec,nvec)
c
        if (fval .lt. fmin) then
          fmin=fval
          do i1=1,nvec
            xmin(i1)=xvec(i1)
          enddo
c
c Print update
c
          if (iprint .gt. 1) then
            write (6,602) fmin,iloop
          endif
c
        endif
c
      enddo
c
c Copy minimum back to xvec
c
      do i1=1,nvec
        xvec(i1)=xmin(i1)
      enddo
c
c Save random number seed
c
      value=float(iseed)
      call putop('RANUNI.ISEED',value)
c
c End
c
 600  format (' monte: v1.2')
 601  format (' monte:   initial function value = ',
     +        1pe15.5)
 602  format (' monte:   improved minimum value = ',
     +        1pe15.5,' at iteration = ',i6)
c
 699  format (' *** Error: monte: parameter vector too large ***')
c
      end
