*
      subroutine powll(func,xvlo,xvec,xvhi,nvec)
*
* powll: Powell-method minimization: uses the NR subroutines of
*        Press et al, with a slight style modification.
*
* Update history
*    v1.0   8-Oct-1992  Original version
*    v1.1  16-Jun-1994  Change name of search subroutine
*    v1.2  17-Jul-1998  Small improvements
*    v1.3  24-Sep-1999  Changed meaning of iprint levels
*
      implicit none
*
* Arguments
*    func = r*4 function = quantity to minimize
*    xvlo = r*4(nvec)    = array of lower limits to xvec
*    xvec = r*4(nvec)    = array of quantities to find at minimum
*    xvhi = r*4(nvec)    = array of upper limits to xvec
*    nvec = i*4          = number of parameters
*
      integer*4 nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
*
* Subprogrammes
*    lnmnop = subroutine = modified Press et al routine
*
      external lnmnop
*
* Local parameter
*    NVMAX = i*4 = maximum length of vectors
*
      integer*4  NVMAX
      parameter (NVMAX=128)
*
* Local variables
*    del    = r*4              = amount of function change
*    eps    = r*4              = fractional accuracy demanded
*    fmin   = r*4              = minimum of function
*    fp     = r*4              = value of function at current point
*    fptt   = r*4              = value at previous point
*    i1     = i*4              = counting index
*    i2     = i*4              = counting index
*    ibig   = i*4              = direction of largest change
*    iprint = i*4              = print level
*    iter   = i*4              = number of iterations used
*    nimax  = i*4              = maximum allowed iterations
*    test   = r*4              = test quantity in method
*    ptt    = r*4(NVMAX)       = previous parameters
*    xini   = r*4(NVMAX)       = initial search direction
*    xit    = r*4(NVMAX)       = amount of shift
*    xmat   = r*4(NVMAX,NVMAX) = matrix used by POWELL
*
      real*4    del,eps,fmin,fp,fptt,test
      real*4    ptt(NVMAX),xini(NVMAX),xit(NVMAX),xmat(NVMAX,NVMAX)
      integer*4 i1,i2,ibig,iprint,iter,nimax
*
* Get controlling variables
*
      call getop('POWLL.EPS'   ,eps )
      call getop('POWLL.NIMAX' ,test)
      nimax =ifix(test+0.5)
      call getop('POWLL.IPRINT',test)
      iprint=ifix(test+0.5)
*
* Title
*
      if (iprint .gt. 0) then
        write (6,600)
      endif
*
* Check that vector length isn't too large
*
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
*
* Initialise matrix of search directions
*
      do i1=1,nvec
        do i2=1,nvec
          xmat(i2,i1)=0.0
        enddo
        xmat(i1,i1)=1.0
      enddo
*
* Use initial vector to get initial minimum; save initial vector
*
      fmin=func(xvec,nvec)
      do i1=1,nvec
        xini(i1)=xvec(i1)
      enddo
*
      if (iprint .gt. 0) then
        write (6,601) fmin
      endif
*
* Initialise counter of interations
*
      iter=0
*
* Main loop, done once per iteration
*
 1    continue
        iter=iter+1
*
* Save current value of the minimum, and initialise the pointer
* to the `best' direction in which to do the minimum (ibig). Also
* initialise del, the largest value of the decrease in the function.
*
        fp  =fmin
        ibig=0
        del =0.0
*
* Loop over directions in the matrix, stepping once in each of
* the directions to locate the minimum.
*
        do i1=1,nvec
*
* Select direction into vector xit
*
          do i2=1,nvec
            xit(i2)=xmat(i2,i1)
          enddo
*
* Find the minimum in the function in direction xit, allowing 
* motion only within the legal bounds.
*
          call lnmnop(func,xvlo,xvec,xvhi,xit,nvec,fmin)
*
* Record the direction index which gives the largest decrease
* in the function
*
          if (abs(fp-fmin) .gt. del) then
            del =abs(fp-fmin)
            ibig=i1
          endif
        enddo
*
* Done if the function decrease is less than the tolerance
*
        if (abs(fp-fmin) .le. 0.5*eps*(abs(fp)+abs(fmin))) then
*
          if     (iprint .gt. 1) then
            write (6,602) fmin,iter
            write (6,603) (xvec(i1), i1=1,nvec)
          elseif (iprint .gt. 0) then
            write (6,602) fmin,iter
          endif
*
          return
*
        endif
*
* If the number of iterations has reached the limit, then done
*
        if (iter .ge. nimax) then
          if (iprint .gt. 2) then
            write (6,604) fmin,iter
          endif
          return
        endif
*
* Construct the extrapolated point to which the minimum moved,
* and store as the initial seach point for the next loop.
*    
        do i2=1,nvec
          ptt(i2) =max(xvlo(i2),
     +                 min(2.0*xvec(i2)-xini(i2),xvhi(i2)))
          xit(i2) =    xvec(i2)-xini(i2)
          xini(i2)=    xvec(i2)
        enddo
*
* Get value of function at extrapolated point beyond minimum
*
        fptt=func(ptt,nvec)
*
* If this value is greater than the original value at the start
* of the loop, then don't include the new direction in the search
*
        if (fptt .ge. fp) then
          if (iprint .gt. 2) then
            write (6,605) fmin,iter
          endif
          goto 1
        endif
*
* If near bottom of minimum, or the average direction is not 
* particularly good, don't include new direction in search
*
        test=2.0*(fp-2.0*fmin+fptt)*(fp-fmin-del)**2
     +       -del*(fp-fptt)**2
        if (test .ge. 0.0) then
          if (iprint .gt. 2) then
            write (6,606) fmin,iter
          endif
          goto 1
        endif
*
* Locate the minimum in the new direction created from the search
*
        call lnmnop(func,xvlo,xvec,xvhi,xit,nvec,fmin)
*
* Save the new direction into the search matrix to replace the
* previous best direction
*
        do i2=1,nvec
          xmat(i2,ibig)=xit(i2)
        enddo
*
      if (iprint .gt. 2) then
        write (6,607) fmin,iter
      endif
*
* Return to top of loop
*
      goto 1
*
 600  format (' powll: v1.2')
 601  format (' powll: initial function value = ',
     +        1pe15.5)
 602  format (' powll:   converged to minimum = ',
     +        1pe15.5,' at iteration = ',i6)
 603  format (' powll:        improved vector = ',
     +        4(1pe15.5),31(/' ',32x,4(1pe15.5)))
 604  format (' powll: iteration limit, value = ',
     +        1pe15.5,' at iteration = ',i6)
 605  format (' powll:   (failed fptt), value = ',
     +        1pe15.5,' at iteration = ',i6)
 606  format (' powll:   (failed test), value = ',
     +        1pe15.5,' at iteration = ',i6)
 607  format (' powll: (new direction), value = ',
     +        1pe15.5,' at iteration = ',i6)
*
 699  format (' *** Error: powll: parameter vector too large ***')
*
      end
