c
      subroutine siman(func,xvlo,xvec,xvhi,nvec)
c
c siman: Use simulated annealing to minimise the function func 
c        of parameters xvec(1-nvec) in range xvlo < xvec < xvhi
c
c Update history
c    V1.0  15-Oct-1992  Original version
c    V2.0  14-Dec-1992  Add reading parameters; change variables
c    V2.2  17-Jun-1994  Add print on request
c    V2.3  20-Jul-1998  Improved treatment
c
      implicit none
c
c Arguments
c    func = r*4 function = quantity to minimize
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c
      integer*4 nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
c
c Local parameters
c    NVMAX = i*4 = maximum size of vectors
c
      integer*4  NVMAX
      parameter (NVMAX=128)
c
c Subprogrammes
c    getop  = subroutine   = get controlling parameters
c    ranuni = r*4 function = random number generator
c
      real*4   ranuni
      external getop,ranuni
c
c Local variables
c    ex      = r*4        = protected Boltzmann factor
c    fmin    = r*4        = minimum of function
c    fval    = r*4        = current value of function
c    i1      = i*4        = counting index
c    i2      = i*4        = counting index
c    i3      = i*4        = counting index
c    indx    = i*4        = index of variable to change
c    ipnn(i) = i*4(NVMAX) = list of changing variable indices
c    iprint  = i*4        = print level
c    iseed   = i*4        = seed for random number generator
c    nchn    = i*4        = counter of changes made
c    nloop   = i*4        = maximum number of temperatures to try
c    npnn    = i*4        = number of non-null parameters
c    nsamp   = i*4        = number of movements at each temperature
c    tchn    = r*4        = factor for temperature reduction each cycle
c    temp    = r*4        = current temperature
c    tiny    = r*4        = smallest temperature
c    vprev   = r*4        = previous value of parameter being varied
c    xl      = r*4        = log10(current temperature)
c    xcur(i) = r*4(NVMAX) = current work vector
c
      real*4    ex,fmin,fval,tchn,temp,tiny,vprev,xl
      real*4    xcur(NVMAX)
      integer*4 i1,i2,i3,iseed,indx,nchn,nloop,nsamp
      integer*4 ipnn(NVMAX),iprint,npnn
c
c Get control parameters
c
      call getop('SIMAN.TCHN' ,tchn)
      call getop('SIMAN.TINY' ,tiny)
      call getop('SIMAN.NLOOP',temp)
      nloop=ifix(temp+0.5)
      call getop('SIMAN.NSAMP',temp)
      nsamp=ifix(temp+0.5)
      call getop('SIMAN.IPRINT',temp)
      iprint=ifix(temp+0.5)
c
c Get random number generator seed
c
      call getop('RANUNI.ISEED',temp)
      iseed=ifix(temp+0.5)
c
c Title
c
      if (iprint .gt. 0) then
        write (6,600)
      endif
c
c Check vector length
c
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
c
c Save initial and current values of function
c
      fmin=func(xvec,nvec)
c
c Copy input vector to current work vector
c
      do i1=1,nvec
        xcur(i1)=xvec(i1)
      enddo
c
c Find number of non-null parameters and store their indices
c
      npnn=0
      do i1=1,nvec
        if (xvlo(i1) .lt. xvhi(i1)) then
          npnn      =npnn+1
          ipnn(npnn)=i1
        endif
      enddo
c
c Print value at start
c
      if (iprint .gt. 0) then
        write (6,601) fmin
      endif
c
c Set initial value of temperature variable
c
      temp=2.0*(abs(fmin)+1.0)
c
c Main loop, try a large number of temperature settings 
c
      do i1=1,nloop
c
c Get new temperature and initialise counter of successful
c    changes. Note underflow protection: exit the loop if
c    the temperature is too small.
c
        xl  =alog10(tchn)+alog10(temp)
        if (xl .lt. alog10(TINY)) then
          if (iprint .gt. 0) then
            write (6,602) fmin,i1
          endif
          return
        endif
        temp=10.0**xl
        nchn=0
c
c At this temperature, try a large number of new values 
c    for the parameters, taken singly
c
        do i2=1,nsamp
c
c Decide which variable to change
c
          indx=ipnn(max0(1,
     +                   min0(1+ifix(FLOAT(npnn)*ranuni(iseed)),
     +                        npnn)
     +                  )
     +             )
c
c Decide new value to try; insert into vector, saving old
c    value, and evaluate function
c
          vprev     =xcur(indx)
          xcur(indx)=xvlo(indx)+ranuni(iseed)*(xvhi(indx)-xvlo(indx))
          fval      =func(xcur,nvec)
c
c Accept change in value if either fval is less than present fmin or,
c    with probability exp(-(fval-fmin)/temp), even if it's worse.
c    Otherwise return variable to previous state.
c
          xl=(fval-fmin)/temp
          if (xl .lt. 25.0) then
            ex=exp(-xl)
          else
            ex=0.0
          endif
c
          if     (fval .lt. fmin) then
            fmin=fval
            nchn=nchn+1
            do i3=1,nvec
              xvec(i3)=xcur(i3)
            enddo
          elseif (ranuni(iseed) .lt. ex) then
            nchn=nchn+1
          else
            xcur(indx)=vprev
          endif
c
c Loop back to change another variable
c
        enddo
c
c If no changes occurred at this temperature, then abandon hope
c
        if (nchn .le. 0) then
          if (iprint .gt. 0) then
            write (6,603) fmin,i1
          endif
          return
        else
          if (iprint .gt. 1) then
            write (6,604) fmin,i1
          endif
        endif
c
c End of main loop
c
      enddo
c
c Reached end of annealing schedule, but without reaching a
c   solution
c
      if (iprint .gt. 0) then
        write (6,605) fmin,nloop
      endif
c
c Save random number seed
c
      temp=float(iseed)
      call putop('RANUNI.ISEED',temp)
c
c End
c
 600  format (' siman: v2.3')
 601  format (' siman:   initial function value = ',
     +        1pe15.5)
 602  format (' siman:    minimum temp at value = ',
     +        1pe15.5,' at iteration = ',i6)
 603  format (' siman:     converged to minimum = ',
     +        1pe15.5,' at iteration = ',i6)
 604  format (' siman:  still changing at value = ',
     +        1pe15.5,' at iteration = ',i6)
 605  format (' siman: iteration limit at value = ',
     +        1pe15.5,' at iteration = ',i6)
c
 699  format (' *** Error: siman: vector too long ***')
c
      end
