c
      subroutine simmu(func,xvlo,xvec,xvhi,nvec)
c
c simmu: Use simulated annealing to minimise the function func
c        of parameters xvec(1-nvec) in range xvlo < xvec < xvhi. 
c
c Update history
c    V1.0  15-Oct-1992  Original version (SIMAN)
c    V2.0  16-Oct-1992  Anneal multiple points also (SIMMU)
c    V2.2  17-Jun-1994  Add printing options
c    V2.3  20-Jul-1998  Small improvements
c
      implicit none
c
c Arguments
c    func = r*4 function = quantity to minimize
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c
      integer*4 nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
c
c Local parameters
c    NVMAX = i*4 = maximum size of vectors
c
      integer*4  NVMAX
      parameter (NVMAX=128)
c
c Subprogrammes
c    ranuni = r*4 function = random number generator
c    getop  = subroutine   = get values of control variables
c
      real*4   ranuni
      external getop,ranuni
c
c Local variables
c    ex      = r*4        = protected Boltzmann factor
c    fmin    = r*4        = minimum of function
c    fval    = r*4        = current value of function
c    i1      = i*4        = counting index
c    i2      = i*4        = counting index
c    i3      = i*4        = counting index
c    indx    = i*4        = index of variable to change
c    ipnn(i) = i*4(NVMAX) = list of changing variable indices
c    iprint  = i*4        = print level
c    iseed   = i*4        = random number seed
c    nchn    = i*4        = counter of changes made
c    nloop   = i*4        = maximum number of temperatures to try
c    npnn    = i*4        = number of non-null parameters
c    nsamp   = i*4        = number of movements at each temperature
c    nvc     = i*4        = number of variables to change
c    tchn    = r*4        = factor for temperature reduction each cycle
c    temp    = r*4        = current temperature
c    tiny    = r*4        = minimum temperature
c    xl      = r*4        = log10(current temperature)
c    xcur(i) = r*4(NVMAX) = current work vector
c    xprv(i) = r*4(NVMAX) = work vector before changing come parameters
c
      real*4    ex,fmin,fval,tchn,temp,tiny,xl
      real*4    xcur(NVMAX),xprv(NVMAX)
      integer*4 i1,i2,i3,indx,iprint,iseed,nchn,nloop,npnn,nsamp,nvc
      integer*4 ipnn(NVMAX)
c
c Check vector length
c
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
c
c Get values of control variables
c
      call getop('SIMMU.TCHN' ,tchn)
      call getop('SIMMU.TINY' ,tiny)
      call getop('SIMMU.NLOOP',temp)
      nloop=ifix(temp+0.5)
      call getop('SIMMU.NSAMP',temp)
      nsamp=ifix(temp+0.5)
      call getop('SIMMU.IPRINT',temp)
      iprint=ifix(temp+0.5)
c
c Get random number generator seed
c
      call getop('RANUNI.ISEED',temp)
      iseed=ifix(temp+0.5)
c
c Save initial and current values of function
c
      fmin=func(xvec,nvec)
c
c Copy input vector to current work vector
c
      do i1=1,nvec
        xcur(i1)=xvec(i1)
      enddo
c
c Find number of non-null parameters and store their indices
c
      npnn=0
      do i1=1,nvec
        if (xvlo(i1) .lt. xvhi(i1)) then
          npnn      =npnn+1
          ipnn(npnn)=i1
        endif
      enddo
c
c Print value at start
c
      if (iprint .gt. 0) then
        write (6,601) fmin
      endif
c
c Set initial value of temperature variable
c
      temp=2.0*(abs(fmin)+1.0)
c
c Main loop, try a large number of temperature settings 
c
      do i1=1,nloop
c
c Get new temperature and initialise counter of successful
c    changes. Note underflow protection: exit the loop if
c    the temperature is too small.
c
        xl  =alog10(tchn)+alog10(temp)
        if (xl .lt. alog10(TINY)) then
          if (iprint .gt. 0) then
            write (6,602) fmin,i1
          endif
          return
        endif
        temp=10.0**xl
        nchn=0
c
c At this temperature, try a large number of new values 
c    for the parameters.
c
        do i2=1,nsamp
c
c Save current vector
c
          do i3=1,nvec
            xprv(i3)=xcur(i3)
          enddo
c
c Decide how many variables to change, and change them. May
c    involve changing one variable several times.
c
          nvc=1+ifix(float(nvec)*ranuni(iseed))
          do i3=1,nvc
            indx=ipnn(max0(1,
     +                     min0(1+ifix(float(npnn)*ranuni(iseed)),
     +                          npnn)
     +                    )
     +               )
            xcur(indx)=xvlo(indx)+ranuni(iseed)*(xvhi(indx)-xvlo(indx))
          enddo
c
c Evaluate function
c
          fval=func(xcur,nvec)
c
c Accept change in value if either fval is less than present fmin or,
c    with probability exp(-(fval-fmin)/temp), even if it's worse.
c    Otherwise return variables to previous state.
c
          xl=(fval-fmin)/temp
          if (xl .lt. 25.0) then
            ex=exp(-xl)
          else
            ex=0.0
          endif
c
          if     (fval .lt. fmin) then
            fmin=fval
            nchn=nchn+1
            do i3=1,nvec
              xvec(i3)=xcur(i3)
            enddo
          elseif (ranuni(iseed) .lt. ex) then
            nchn=nchn+1
          else
            do i3=1,nvec
              xcur(i3)=xprv(i3)
            enddo
          endif
c
c Loop back to change another variable
c
        enddo
c
c If no changes occurred at this temperature, then abandon hope
c
        if (nchn .le. 0) then
          if (iprint .gt. 0) then
            write (6,603) fmin,i1
          endif
          return
        else
          if (iprint .gt. 1) then
            write (6,604) fmin,i1
          endif
        endif
c
c End of main loop
c
      enddo
c
c Reached end of annealing schedule, but without reaching a
c   solution
c
      if (iprint .gt. 0) then
        write (6,605) fmin,nloop
      endif
c
c Save random number generator seed
c
      temp=float(iseed)
      call putop('RANUNI.ISEED',temp)
c
c End
c
 600  format (' simmu: v2.3')
 601  format (' simmu:   initial function value = ',
     +        1pe15.5)
 602  format (' simmu:    minimum temp at value = ',
     +        1pe15.5,' at iteration = ',i4)
 603  format (' simmu:     converged to minimum = ',
     +        1pe15.5,' at iteration = ',i4)
 604  format (' simmu:  still changing at value = ',
     +        1pe15.5,' at iteration = ',i4)
 605  format (' simmu: iteration limit at value = ',
     +        1pe15.5,' at iteration = ',i4)
c
 698  format (' *** Error: simmu: indexing        ***')
 699  format (' *** Error: simmu: vector too long ***')
c
      end
