c
      subroutine simpu(func,xvlo,xvec,xvhi,nvec)
c
c simpu: Use simulated annealing combined with Powell minimization
c        to minimise the function func of parameters xvec(1-nvec)
c        in range xvlo < xvec < xvhi, and allowing multiple-switches
c
c Update history
c    V1.0  15-Oct-1992  Original version
c    V1.1  10-Aug-1993  Tidied up
c    V1.2  17-Jun-1994  Printing parameter set 
c    V1.3  13-Jul-1998  Improved treatment
c
      implicit none
c
c Arguments
c    func = r*4 function = quantity to minimize
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c
      integer*4 nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
c
c Local parameters
c    NVMAX = i*4 = maximum size of vectors
c
      integer*4  NVMAX
      parameter (NVMAX=128)
c
c Subprogrammes
c    ranuni = r*4 function = random number generator
c    getop  = subroutine   = read control variables
c
      real*4   ranuni
      external getop,ranuni
c
c Local variables
c    ex      = r*4        = protected Boltzmann factor
c    fmin    = r*4        = minimum of function
c    fmnn    = r*4        = local minimum of function
c    fval    = r*4        = current value of function
c    i1      = i*4        = counting index
c    i2      = i*4        = counting index
c    i3      = i*4        = counting index
c    iann    = i*4        = anneal counter
c    iloop   = i*4        = current anneal progress counter
c    indx    = i*4        = index of variable to change
c    ipnn(i) = i*4(NVMAX) = list of changing variable indices
c    iprint  = i*4        = print level
c    iseed   = i*4        = seed for random number generator
c    nanne   = i*4        = number of anneals
c    nchn    = i*4        = counter of changes made
c    nloop   = i*4        = maximum number of temperatures to try
c    npnn    = i*4        = number of non-null parameters
c    nsamp   = i*4        = number of movements at each temperature
c    nvch    = i*4        = number of parameters to change
c    tchn    = r*4        = initial temperature reduction factor
c    temp    = r*4        = current temperature
c    tfac    = r*4        = current anneal cycle rate of T change
c    tiny    = r*4        = limit to value of temperature
c    xl      = r*4        = log10(current temperature)
c    xcur(i) = r*4(NVMAX) = current work vector
c    xvmn(i) = r*4(NVMAX) = local minimum vector
c    xprv(i) = r*4(NVMAX) = previous work vector
c
      real*4    ex,fmin,fmnn,fval,tchn,temp,tfac,tiny,xl
      real*4    xcur(NVMAX),xvmn(NVMAX),xprv(NVMAX)
      integer*4 i1,i2,i3,iann,iloop,indx,iprint,iseed,
     +          nanne,nchn,nloop,npnn,nsamp,nvch
      integer*4 ipnn(NVMAX)
c
c Get control variables
c
      call getop('SIMPU.TCHN'  ,tchn)
      call getop('SIMPU.TINY'  ,tiny)
      call getop('SIMPU.NANNE' ,temp)
      nanne =ifix(temp+0.5)
      call getop('SIMPU.NLOOP' ,temp)
      nloop =ifix(temp+0.5)
      call getop('SIMPU.NSAMP' ,temp)
      nsamp =ifix(temp+0.5)
      call getop('SIMPU.IPRINT',temp)
      iprint=ifix(temp+0.5)
c
c Get current random number seed
c
      call getop('RANUNI.ISEED',temp)
      iseed=ifix(temp+0.5)
c
c Title
c
      if (iprint .gt. 0) then
        write (6,600)
      endif
c
c Check vector length
c
      if (nvec .gt. NVMAX) then
        write (6,699)
        return
      endif
c
c Save initial value of function
c
      fmin=func(xvec,nvec)
c
c Find number of non-null parameters and store their indices
c
      npnn=0
      do i1=1,nvec
        if (xvlo(i1) .lt. xvhi(i1)) then
          npnn      =npnn+1
          ipnn(npnn)=i1
        endif
      enddo
c
c Print value at start
c
      if (iprint .gt. 0) then
        write (6,601) fmin
      endif
c
c Anneal loops
c
      do iann=1,nanne
c
c Initialise count of number of changes
c
        nchn=nloop
c
c Randomize to fill current work vector and local minimum energy
c    and current best vector in this anneal programme
c
        do i1=1,nvec
          xvmn(i1)=xvlo(i1)+ranuni(iseed)*(xvhi(i1)-xvlo(i1))
          xcur(i1)=xvmn(i1)
        enddo
        fmnn=func(xcur,nvec)
c
c Message about start of new anneal loop
c
        if (iprint .gt. 1) then
          write (6,602) fmnn,iann
        endif
c
c Set initial value of temperature variable for this anneal
c    cycle
c
        temp=(2.0-1.5*(float(iann-1)/float(nanne)))*(abs(fmin)+1.0)
c
c Set anneal rate for this cycle
c
        tfac=tchn*(1.0-0.5*(float(iann-1)/float(nanne)))
c
c Temperature loop, try a large number of settings 
c
        iloop=0
        dowhile ((nchn .gt. 0) .and. (iloop .le. nloop))
          iloop=iloop+1
c
c Get new temperature and initialise counter of successful
c    changes. Note underflow protection: exit the loop if
c    the temperature is too small.
c
          if (tfac .gt. 0.0) then
            xl  =alog10(tfac)+alog10(temp)
            if (xl .gt. alog10(tiny)) then
              temp=10.0**xl
              nchn=0
c
c At this temperature, try a large number of new values 
c    for the parameters, taken singly
c
              do i2=1,nsamp
c
c Save current vector
c
                do i3=1,nvec
                  xprv(i3)=xcur(i3)
                enddo
c
c Decide how many variables to change
c
                nvch=1+ifix(float(nvec)*ranuni(iseed))
c
c Change variables. May involve changing one variable several times
c
                do i3=1,nvch
                  indx=ipnn(max0(1,
     +                      min0(1+ifix(float(npnn)*ranuni(iseed)),
     +                           npnn)
     +                          )
     +                     )
                  xcur(indx)=xvlo(indx)
     +                       +ranuni(iseed)*(xvhi(indx)-xvlo(indx))
                enddo
c
c Evaluate function
c
                fval      =func(xcur,nvec)
c
c Accept change in value if either fval is less than present fmin or,
c    with probability exp(-(fval-fmnn)/temp), even if it's worse.
c    Otherwise return vector to previous state.
c
                xl=(fval-fmnn)/temp
                if (xl .lt. 25.0) then
                  ex=exp(-xl)
                else
                  ex=0.0
                endif
c
                if     (fval .lt. fmnn) then
                  nchn=nchn+1
                  do i3=1,nvec
                    xvmn(i3)=xcur(i3)
                  enddo
                  fmnn=fval
                elseif (ranuni(iseed) .lt. ex) then
                  nchn=nchn+1
                else
                  do i3=1,nvec
                    xcur(i3)=xprv(i3)
                  enddo
                endif
c
c Loop back to change another variable
c
              enddo
c
c End of if statements which check current temperature
c
            endif
c
          endif
c
c End of loop over temperatures
c
        enddo
c
c No further changes: solution has solidified. Try to improve
c     result of anneal loop, using powll-type system
c
        call powll(func,xvlo,xvmn,xvhi,nvec)
        fmnn=func(xvmn,nvec)
c
        if (iprint .gt. 1) then
          write (6,603) fmnn,iann
        endif
c
c Evaluate new result, give current result
c
        if (fmnn .lt. fmin) then
          fmin=fmnn
          do i1=1,nvec
            xvec(i1)=xvmn(i1)
          enddo
        endif
c
        if (iprint .gt. 1) then
          write (6,604) fmin,iann
        endif
c
c Try to improve using powll-type minimization
c
        call powll(func,xvlo,xvec,xvhi,nvec)
        fmin=func(xvec,nvec)
c
c End of anneal loop
c
        if (iprint .gt. 1) then
          write (6,605) fmin,iann
        endif
c
      enddo
c
c Reached end of annealing schedules
c
      if (iprint .gt. 0) then
        write (6,606) fmin,nanne
      endif
c
c Store current random number seed
c
      temp=float(iseed)
      call putop('RANUNI.ISEED',temp)
c
c End
c
 600  format (' simpu: v1.3')
 601  format (' simpu:   initial function value = ',
     +        1pe15.5)
 602  format (' simpu:      new anneal at value = ',
     +        1pe15.5,' at iteration = ',i6)
 603  format (' simpu:     anneal ends at value = ',
     +        1pe15.5,' at iteration = ',i6)
 604  format (' simpu:      unchanging at value = ',
     +        1pe15.5,' at iteration = ',i6)
 605  format (' simpu:   annealing end at value = ',
     +        1pe15.5,' at iteration = ',i6)
 606  format (' simpu:   final minimum at value = ',
     +        1pe15.5,' at iteration = ',i6)
c
 699  format (' *** Error: simpu: vector too long ***')
c
      end
