c
      subroutine smplx(func,xvlo,xvec,xvhi,nvec)
c
c smplx: Simplex-method minimization, using slight modifications
c        of Press et al routines
c
c Update history
c    V1.0  15-Oct-1992  Original version
c    V1.1   4-Aug-1993  Tidy up
c    V1.2  17-Jun-1994  Add printing variable
c    V1.3  17-Jul-1998  Small improvements
c
      implicit none
c
c Arguments
c    func = r*4 function = quantity to minimize
c    xvlo = r*4(nvec)    = array of lower limits to xvec
c    xvec = r*4(nvec)    = array of quantities to find at minimum
c    xvhi = r*4(nvec)    = array of upper limits to xvec
c    nvec = i*4          = number of parameters
c
      integer*4 nvec
      real*4    func
      external  func
      real*4    xvlo(nvec),xvec(nvec),xvhi(nvec)
c
c Subprogrammes
c    getop = subroutine = get values of controlling variables
c
      external getop
c
c Local parameter
c    NVMAX  = i*4 = maximum length of vectors
c
      integer*4  NVMAX
      parameter (NVMAX =128)
c
c Local variables
c    alpha     = r*4              = algorithm convergence factor
c    beta      = r*4              = algorithm convergence factor
c    epssm     = r*4              = fractional accuracy demanded
c    fmin      = r*4              = minimum of function
c    fvec(i)   = r*4(NVMAX)       = function values at vertices
c    gamma     = r*4              = algorithm convergence factor
c    i1        = i*4              = counting index
c    i2        = i*4              = counting index
c    ihi       = i*4              = index of worst vertex
c    ilo       = i*4              = index of best vertex
c    inhi      = i*4              = new worst vertex index
c    iprint    = i*4              = print level
c    iter      = i*4              = number of iterations used
c    mpts      = i*4              = needed size of matrix
c    nimax     = i*4              = maximum number of iterations
c    pbar(i)   = r*4(NVMAX)       = average of simplex vertices
c    pr(i)     = r*4(NVMAX)       = new test vector
c    prr(i)    = r*4(NVMAX)       = revised new test vector
c    xmat(i,j) = r*4(NVMAX,NVMAX) = matrix of vertex positions
c    ypr       = r*4              = function value at pr
c    yprr      = r*4              = function value at prr
c
      real*4    fvec(NVMAX),pr(NVMAX),prr(NVMAX),pbar(NVMAX)
      real*4    xmat(NVMAX,NVMAX)
      real*4    alpha,beta,epssm,fmin,gamma,rtol,ypr,yprr
      integer*4 i1,i2,ihi,ilo,inhi,iprint,iter,mpts,nimax
c
c Get controlling variables
c
      call getop('SMPLX.ALPHA',alpha)
      call getop('SMPLX.BETA' ,beta)
      call getop('SMPLX.GAMMA',gamma)
      call getop('SMPLX.EPS'  ,epssm)
      call getop('SMPLX.NIMAX',rtol)
      nimax =ifix(rtol+0.5)
      call getop('SMPLX.IPRINT',fmin)
      iprint=ifix(fmin+0.5)
c
c Title
c
      if (iprint .gt. 0) then
        write (6,600)
      endif
c
c Calculate size of matrix needed for minimization
c
      mpts=nvec+1
c
c Check that size isn't too large
c
      if (mpts .gt. NVMAX) then
        write (6,699)
        return
      endif
c
c Initialise matrix of starting vertices of simplex: the initial
c    vector of parameters and the unit vectors in the other variables
c    
      do i2=1,nvec
        xmat(1,i2)=xvec(i2)
      enddo
      do i1=2,nvec+1
        do i2=1,nvec
          xmat(i1,i2)=0.0
        enddo
        xmat(i1,i1-1)=1.0
      enddo
c
c Initialise vector of starting values of function on vertices
c    of simplex
c
      do i1=1,nvec+1
        do i2=1,nvec
          prr(i2)=max(xvlo(i2),min(xmat(i1,i2),xvhi(i2)))
        enddo
        fvec(i1)=func(prr,nvec)
      enddo
c
c Print value at start
c
      if (iprint .gt. 0) then
        write (6,601) fvec(1)
      endif
c
c Recoding of the AMOEBA routine from Press et al:
c    main loop covered once per iteration
c
      iter=0
 1    continue
c
c Determine vectors to highest and lowest values of function,
c    as marked by indices ilo and ihi
c
        ilo=1
        if (fvec(1) .gt. fvec(2)) then
          ihi =1
          inhi=2
        else
          ihi =2
          inhi=1
        endif
        do i1=1,mpts
          if (fvec(i1) .lt. fvec(ilo)) then
            ilo=i1
          endif
          if     (fvec(i1) .gt. fvec(ihi)) then
            inhi=ihi
            ihi =i1
          elseif (fvec(i1) .gt. fvec(inhi)) then
            if (i1 .ne. ihi) then
              inhi=i1
            endif
          endif
        enddo
c
c Copy best result so far to vector xvec
c
        do i2=1,nvec
          xvec(i2)=max(xvlo(i2),min(xmat(ilo,i2),xvhi(i2)))
        enddo
        fmin=fvec(ilo)
c
c Compute fractional range from highest to lowest point of simplex.
c    
        rtol=2.0*abs(fvec(ihi)-fvec(ilo))/
     +          (abs(fvec(ihi))+abs(fvec(ilo)))
c
c If the range is small enough, return
c
        if (rtol .lt. epssm) then
          if (iprint .gt. 0) then
            write (6,602) fmin,iter
          endif
          return
        endif
c
c Check number of iterations used; if at maximum return
c
        if (iter .eq. nimax) then
          if (iprint .gt. 0) then
            write (6,603) fmin,iter
          endif
          return
        endif
c
c Increment iteration counter
c        
        iter=iter+1
c
c Find vector average of all points except the worst, pbar, and the
c    extrapolation by a factor alpha from the high point through
c    the mean (a reflection about the mean), pr
c
        do i2=1,nvec
          pbar(i2)=0.0
        enddo
c
        do i1=1,mpts
          if (i1 .ne. ihi) then
            do i2=1,nvec
              pbar(i2)=pbar(i2)+xmat(i1,i2)
            enddo
          endif
        enddo
c
        do i2=1,nvec
          pbar(i2)=pbar(i2)/FLOAT(nvec)
          pbar(i2)=max(xvlo(i2),min(pbar(i2),xvhi(i2)))
          pr(i2)  =(1.0+alpha)*pbar(i2)-alpha*xmat(ihi,i2)
          pr(i2)  =max(xvlo(i2),min(pr(i2),xvhi(i2)))
        enddo
c
c Find the function value at the reflected point
c
        ypr=func(pr,nvec)
c
c If this has improved matters to make pr the best simplex
c    vertex, try a further extrapolation by an amount gamma
c    to make the vector prr
c
        if     (ypr .le. fvec(ilo)) then
          do i2=1,nvec
            prr(i2)=gamma*pr(i2)+(1.0-gamma)*pbar(i2)
            prr(i2)=max(xvlo(i2),min(prr(i2),xvhi(i2)))
          enddo
          yprr=func(prr,nvec)
c
c If the additional extrapolation worked, use it to replace the
c    worst point in the simplex. Otherwise, replace the high
c    point by the pr result.
c
          if (yprr .lt. fvec(ilo)) then
            do i2=1,nvec
              xmat(ihi,i2)=prr(i2)
            enddo
            fvec(ihi)=yprr
            if (iprint .gt. 1) then
              write (6,604) fmin,iter
            endif
          else
            do i2=1,nvec
              xmat(ihi,i2)=pr(i2)
            enddo
            fvec(ihi)=ypr
            if (iprint .gt. 1) then
              write (6,605) fmin,iter
            endif
          endif
c
c If the reflection did not improve matters, look to see if the
c    reflected point is better than the second highest
c
        elseif (ypr .ge. fvec(inhi)) then
c
c Reflected point is worse than second highest. Replace the highest
c    point if it's better than the highest
c
          if (ypr .lt. fvec(ihi)) then
            do i2=1,nvec
              xmat(ihi,i2)=pr(i2)
            enddo
            fvec(ihi)=ypr
            if (iprint .gt. 1) then
              write (6,606) fmin,iter
            endif
          endif
c
c Look for an intermediate better point, using it if it
c    is good enough
c
          do i2=1,nvec
            prr(i2)=beta*xmat(ihi,i2)+(1.0-beta)*pbar(i2)
            prr(i2)=max(xvlo(i2),min(prr(i2),xvhi(i2)))
          enddo
          yprr=func(prr,nvec)
c
          if (yprr .lt. fvec(ihi)) then
            do i2=1,nvec
              xmat(ihi,i2)=prr(i2)
            enddo
            fvec(ihi)=yprr
            if (iprint .gt. 1) then
              write (6,607) fmin,iter
            endif
          else
c
c If highest point cannot be improved on in this way, contract
c    simplex about lowest point by a factor 0.5, recalculating
c    all function values
c
            do i1=1,mpts
              if (i1 .ne. ilo) then
                do i2=1,nvec
                  pr(i2)     =0.5*(xmat(i1,i2)+xmat(ilo,i2))
                  pr(i2)     =max(xvlo(i2),min(pr(i2),xvhi(i2)))
                  xmat(i1,i2)=pr(i2)
                enddo
                fvec(i1)=func(pr,nvec)
              endif
            enddo
            if (iprint .gt. 1) then
              write (6,608) fmin,iter
            endif
c
          endif
c
c Original reflection gives a middling point. Replace old high
c    point and continue
c
        else
          do i2=1,nvec
            xmat(ihi,i2)=pr(i2)
          enddo
          fvec(ihi)=ypr
          if (iprint .gt. 1) then
            write (6,609) fmin,iter
          endif
        endif
c
c End of main loop. Go back and try again.
c
      goto 1
c
 600  format (' smplx: v1.3')
 601  format (' smplx:   initial function value = ',
     +        1pe15.5)
 602  format (' smplx:     converged to minimum = ',
     +        1pe15.5,' at iteration = ',i6)
 603  format (' smplx: iteration limit at value = ',
     +        1pe15.5,' at iteration = ',i6)
 604  format (' smplx: (2-extrapolate) at value = ',
     +        1pe15.5,' at iteration = ',i6)
 605  format (' smplx:  (extrapolated) at value = ',
     +        1pe15.5,' at iteration = ',i6)
 606  format (' smplx:  (nearly worst) at value = ',
     +        1pe15.5,' at iteration = ',i6)
 607  format (' smplx:  (intermediate) at value = ',
     +        1pe15.5,' at iteration = ',i6)
 608  format (' smplx:   (contraction) at value = ',
     +        1pe15.5,' at iteration = ',i6)
 609  format (' smplx:   (mid reflect) at value = ',
     +        1pe15.5,' at iteration = ',i6)
c
 699  format (' *** Error: smplx: parameter vector too large ***')
c
      end
